@@ -25,8 +25,8 @@ static SEXP get_ada_field(ada_string val){
 
 SEXP R_parse_url(SEXP url, SEXP baseurl) {
   ada_url *result = Rf_length(baseurl) == 0 ?
-    ada_parse(get_string(url), Rf_length(STRING_ELT(url, 0))) :
-    ada_parse_with_base(get_string(url), Rf_length(STRING_ELT(url, 0)), get_string(baseurl), Rf_length(STRING_ELT(baseurl, 0)));
+    ada_parse(get_string(url), len_string(url)) :
+    ada_parse_with_base(get_string(url), len_string(url), get_string(baseurl), len_string(baseurl));
   if(!ada_is_valid(result)){
     ada_free(result);
     Rf_error("ADA failed to parse URL: %s", get_string(url));
@@ -48,6 +48,31 @@ SEXP R_parse_url(SEXP url, SEXP baseurl) {
   return out;
 }
 
+#define set_ada_value(val, fun) if(Rf_length(val)) fun(result, get_string(val), len_string(val))
+
+SEXP R_modify_url(SEXP url, SEXP scheme, SEXP host, SEXP port, SEXP path,
+                 SEXP query, SEXP fragment, SEXP user, SEXP password){
+  if(!Rf_length(url))
+    Rf_error("Either URL or scheme_host are required");
+  ada_url *result = ada_parse(get_string(url), len_string(url));
+  set_ada_value(url, ada_set_href);
+  set_ada_value(scheme, ada_set_protocol);
+  set_ada_value(host, ada_set_hostname);
+  set_ada_value(port, ada_set_port);
+  set_ada_value(path, ada_set_pathname);
+  set_ada_value(query, ada_set_search);
+  set_ada_value(fragment, ada_set_hash);
+  set_ada_value(user, ada_set_username);
+  set_ada_value(password, ada_set_password);
+  if(!ada_is_valid(result)){
+    ada_free(result);
+    Rf_error("ADA failed to build URL");
+  }
+  SEXP out = get_ada_field(ada_get_href(result));
+  ada_free(result);
+  return out;
+}
+
 #elif defined(HAS_CURL_PARSER)
 static void fail_if(CURLUcode err){
   if(err != CURLUE_OK)
@@ -101,6 +126,35 @@ SEXP R_parse_url(SEXP url, SEXP baseurl) {
   UNPROTECT(1);
   return out;
 }
+
+static void set_value(CURLU *h, CURLUPart part, SEXP value){
+  if(Rf_length(value)){
+    if(Rf_inherits(value, "AsIs")){
+      fail_if(curl_url_set(h, part, get_string(value), 0));
+    } else {
+      fail_if(curl_url_set(h, part, get_string(value), CURLU_URLENCODE));
+    }
+  }
+}
+
+SEXP R_modify_url(SEXP url, SEXP scheme, SEXP host, SEXP port, SEXP path, SEXP query, SEXP fragment, SEXP user, SEXP password){
+  CURLU *h = curl_url();
+  set_value(h, CURLUPART_URL, url);
+  set_value(h, CURLUPART_SCHEME, scheme);
+  set_value(h, CURLUPART_HOST, host);
+  set_value(h, CURLUPART_PORT, port);
+  set_value(h, CURLUPART_PATH, path);
+  set_value(h, CURLUPART_QUERY, query);
+  set_value(h, CURLUPART_FRAGMENT, fragment);
+  set_value(h, CURLUPART_USER, user);
+  set_value(h, CURLUPART_PASSWORD, password);
+  char *str = NULL;
+  fail_if(curl_url_get(h, CURLUPART_URL, &str, 0));
+  SEXP out = make_string(str);
+  curl_free(str);
+  return out;
+}
+
 #else
 SEXP R_parse_url(SEXP url, SEXP baseurl) {
   Rf_error("URL parser not suppored, this libcurl is too old");
